import queue
import subprocess
import threading
import sys


def read_from_stream(stream, output_func, q):
    """
    从给定的流中读取数据，并使用指定的函数输出。
    """
    result = []
    for line in iter(stream.readline, b''):
        if line is None:
            break
        decoded_line = line.decode('utf-8').rstrip() if isinstance(line, bytes) else line.rstrip()
        output_func(decoded_line)
        result.append(decoded_line)

    q.put(result)

def print_stdout(line):
    """
    打印标准输出。
    """
    print(line)


def print_stderr(line):
    """
    打印标准错误输出（到stderr）。
    """
    print('stderr', line, file=sys.stderr)


def run_command(command):
    """
    运行命令，并启动两个线程来分别监听stdout和stderr。
    """
    outq = queue.Queue()
    errq = queue.Queue()

    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    # 创建并启动线程
    stdout_thread = threading.Thread(target=read_from_stream, args=(process.stdout, print_stdout, outq))
    stderr_thread = threading.Thread(target=read_from_stream, args=(process.stderr, print_stderr, errq))

    stdout_thread.start()
    stderr_thread.start()

    # 等待子进程完成
    stdout_thread.join(timeout=60)
    stderr_thread.join(timeout=60)

    # 等待子进程完成
    process.wait()

    return process.poll(), outq.get(), errq.get()


command = ['ping', '-c', '5', 'xiexianbin.cn']
print(run_command(command))
